﻿/**
* FlickrBlogs
* last update 08/25/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrBlogs
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _email : String;
	private var _password : String;
	private var _rest_endpoint : String;
	private var _response : XML;
	private var _blogs: Array;
	/**
	* callback function, called at the end of a getList method
	* @see <a href="#getList">getList</a>
	* */
	public var onListLoaded:Function;
	/** 
	* callback function, called at the end of a postPhoto method
	* @see <a href="#postPhoto">postPhoto</a>
	* */
	public var onPhotoPosted:Function;
	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrBlogs, object you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param user_email:	(optional) The email to use on methods that requires user authentication 
	* @param user_password:	(optional) The email to use on methods that requires user authentication 
	* @see <a href="http://www.flickr.com/services/api/misc.userauth.html">User Authentication Flickr API page</a>
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrBlogs (api_key : String, user_email : String, user_password : String,rest_endpoint : String)
	{
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};
		this._response = new XML()
		this._response.ignoreWhite = true;
		this._email = user_email;
		this._password = user_password;
	};
	/**
	* Get a list of configured blogs for the calling user
	* entries that can be accessed through the list attribute
	* @param user_email:	(required) The email used to authenticate on Flickr 
	* @param user_password:	(required) The password used to authenticate on Flickr
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.blogs.onListLoaded = function(error,obj){
	*	if(!error){
	*	 	trace(obj.list[0].name)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.blogs.getList(USEREMAIL,USERPASSWD);
	* </code>
	* @see <a href="#list">list</a>
	* @see <a href="#onListLoaded">onListLoaded</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.blogs.getList.html">flickr.blogs.getList at Flickr API page</a>
	* */
	public function getList (user_email:String,user_password:String)
	{
		var flickrBlogsObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.blogs.getList&api_key="+this._api_key;
		if(user_email) this._email = user_email;
		if(user_password) this._password = user_password;
		if ((this._email)&&(this._password))
		{
			method_url +="&email="+this._email+"&password="+this._password;
		}
		this._response.onLoad = function(success){
			if(success){
				var isRsp = false;
				var blogs = [];
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="blogs"){
								for(var blog = rspChild.firstChild; blog != null; blog = blog.nextSibling){
									if(blog.nodeName=="blog"){
										var entry = new Object();
										entry.id = blog.attributes['id']
										entry.name = blog.attributes['name']
										entry.needspassword = blog.attributes['needspassword']
										entry.url = blog.attributes['url']
										blogs.push(entry)
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrBlogsObjPointer.onListLoaded(error,flickrBlogsObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrBlogsObjPointer.onListLoaded(error,flickrBlogsObjPointer)
				}
				if(!error){
					flickrBlogsObjPointer._blogs = blogs;
					flickrBlogsObjPointer.onListLoaded(0,flickrBlogsObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrBlogsObjPointer.onListLoaded(error,flickrBlogsObjPointer)
			}
		}
		this._response.load(method_url);
	};
	/**
	* Post a photo in the specified blog.
	* @param user_email:		(required) The email used to authenticate on Flickr 
	* @param user_password:		(required) The password used to authenticate on Flickr
	* @param blog_id:			(required) The id of the blog you want to post, you can obtain this id using <a href="#getList">getList</a>
	* @param photo_id:			(required) The id of your flickr photo you want to post on the blog
	* @param post_title:		(required) The title of the post
	* @param description:		(required) A text to go along with the photo on your blog
	* @param blog_password:		(optional) Your blog password, only necessary if you aren't storing it on flickr (needspassword = 1)
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.blogs.onPhotoPosted = function(error){
	* 	if(!error){
	* 		trace("post successful!")
	* 	}else{
	* 		trace(error)
	* 	}
	* }
	* my_flickr.blogs.postPhoto(USEREMAIL,USERPASSWD,BLOGID,PHOTOID,"Some title","A descrition","your_blog_password")
	* </code>
	* @see <a href="#onPhotoPosted">onPhotoPosted</a>"
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.blogs.postPhoto.html">flickr.blogs.postPhoto at Flickr API page</a>
	* */
	public function postPhoto(user_email:String,user_password:String,blog_id:String,photo_id:String,post_title:String,description:String,blog_password:String)
	{
		var flickrBlogsObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.blogs.postPhoto&api_key="+this._api_key+"&blog_id="+blog_id+"&photo_id="+photo_id+"&title="+post_title+"&description="+description;
		if(blog_password){
			method_url += "&blog_password="+blog_password;
		}
		if(user_email) this._email = user_email;
		if(user_password) this._password = user_password;
		if ((this._email)&&(this._password))
		{
			method_url +="&email="+this._email+"&password="+this._password;
			this._response.onLoad = function(success){
				if(success){
					var isRsp = false;
					var error = 0;
					for(var node = this.firstChild; node != null; node = node.nextSibling){
						if(node.nodeName == "rsp"){
							isRsp = true;
							for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
								if(rspChild.nodeName=="err"){
									error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
									flickrBlogsObjPointer.onPhotoPosted(error)
								}
							}
						}
					}
					if(!isRsp){
						error ="ERROR: Response not in REST format";
						flickrBlogsObjPointer.onPhotoPosted(error)
					}
					if(!error){
						flickrBlogsObjPointer.onPhotoPosted(0)
					}					
				}else{
					
				}
			}
			this._response.load(method_url)
		}
		
	}
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}
	/**
	* a list of the blogs, resulted from a getList method call
	* is an Array of objects, each object has the following attributes
	* id, name, needspassword, url
	* @see <a href="#getList">getList</a>
	* */
	public function get list():Array
	{
		return this._blogs;
	}
};
