﻿/**
* FlickrContacts
* last update 08/25/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrContacts
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _email : String;
	private var _password : String;
	private var _rest_endpoint : String;
	private var _response : XML;
	private var _contacts: Array;
	/** 
	* callback function, called at the end of a getList method
	* @see <a href="#getList">getList</a>
	* */
	public var onListLoaded:Function;

	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrContacts object, you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param user_email:	(optional) The email to use on methods that requires user authentication 
	* @param user_password:	(optional) The email to use on methods that requires user authentication 
	* @see <a href="http://www.flickr.com/services/api/misc.userauth.html">User Authentication Flickr API page</a>
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrContacts (api_key:String,user_email:String,user_password:String,rest_endpoint:String)
	{
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};		
		this._response = new XML()
		this._response.ignoreWhite = true;
		this._email = user_email;
		this._password = user_password;
	}
	/**
	* Get a list of contacts for the calling user.
	* entries that can be accessed through the list attribute
	* @param user_email:	(required) The email used to authenticate on Flickr 
	* @param user_password:	(required) The password used to authenticate on Flickr
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.contacts.onListLoaded = function(error,obj){
	*	if(!error){
	*	 	trace(obj.list[0].username)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.contacts.getList(USEREMAIL,USERPASSWD);
	* </code>
	* @see <a href="#list">list</a>
	* @see <a href="#onListLoaded">onListLoaded</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.contacts.getList.html">flickr.contacts.getList at Flickr API page</a>
	* */
	public function getList(){
		var flickrBlogsObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.contacts.getList&api_key="+this._api_key;
		if ((this._email)&&(this._password))
		{
			method_url +="&email="+this._email+"&password="+this._password;
		}
		this._response.onLoad = function(success){
			if(success){
				var isRsp = false;
				var contacts = [];
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="contacts"){
								for(var contact = rspChild.firstChild; contact != null; contact = contact.nextSibling){
									if(contact.nodeName=="contact"){
										var entry = new Object();
										entry.nsid = contact.attributes['nsid']
										entry.username = contact.attributes['username']
										entry.friend = contact.attributes['friend']
										entry.family = contact.attributes['family']
										entry.ignored = contact.attributes['ignored']
										entry.online = contact.attributes['online']
										if(contact.firstChild.nodeType==3){
											//NEED TEST, I'm not certain that it will work
											entry.away_msg = contact.firstChild.nodeValue;
										}
										contacts.push(entry)
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrBlogsObjPointer.onListLoaded(error,flickrBlogsObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrBlogsObjPointer.onListLoaded(error,flickrBlogsObjPointer)
				}
				if(!error){
					flickrBlogsObjPointer._contacts = contacts;
					flickrBlogsObjPointer.onListLoaded(0,flickrBlogsObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrBlogsObjPointer.onListLoaded(error,flickrBlogsObjPointer)
			}
		}
		this._response.load(method_url);
		
	}
	/**
	* a list of the contacts, resulted from a getList method call
	* is an Array of objects, each object has the following attributes
	* nsid, username, friend, family, ignored, online
	* @see <a href="#getList">getList</a>	* */
	public function get list():Array
	{
		return this._contacts;
	}
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}
}