﻿/**
* FlickrFavorites
* last update 08/29/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrFavorites
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _email : String;
	private var _password : String;
	private var _rest_endpoint : String;
	private var _response : XML;
	private var _list: Array;
	/**
	* callback function, called at the end of a add method
	* @see <a href="#addPhoto">addPhoto</a>
	* */
	public var onAdd:Function;
	/** 
	* callback function, called at the end of a getList method
	* @see <a href="#getList">getList</a>
	* */
	public var onGetList:Function;
	/** 
	* callback function, called at the end of a remove method
	* @see <a href="#remove">remove</a>
	* */
	public var onRemove:Function;
	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrBlogs, object you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param user_email:	(optional) The email to use on methods that requires user authentication 
	* @param user_password:	(optional) The email to use on methods that requires user authentication 
	* @see <a href="http://www.flickr.com/services/api/misc.userauth.html">User Authentication Flickr API page</a>
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrFavorites (api_key : String, user_email : String, user_password : String,rest_endpoint : String)
	{
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};
		this._response = new XML()
		this._response.ignoreWhite = true;
		this._email = user_email;
		this._password = user_password;
		this._list = new Array()
	};
	/**
	* Returns a list of the user's favorite photos.
	* entries that can be accessed through the list attribute
	* @param user_email:	(Required) The email used to authenticate on Flickr 
	* @param user_password:	(Required) The password used to authenticate on Flickr
	* @param user_id:		(Optional) The NSID of the user to fetch the favorites list for. 
	* 						If this argument is omitted, the favorites list for the calling user is returned.
	* @param per_page:		(Optional) Number of photos to return per page. If this argument is ommited, it defaults to 100.
	* 						The maximum allowed value is 500.
	* @param page:			(Optional) The page of results to return. If this argument is ommited, it defaults to 1.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.favorites.onGetList = function(error,obj){
	*	if(!error){
	*	 	trace(obj.list[0].id)
	*	 	trace(obj.list[0].owner)
	*	 	trace(obj.list[0].title)
	*	 	trace(obj.list[0].ispublic)
	*	 	trace(obj.list[0].isfriend)
	*	 	trace(obj.list[0].isfamily)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.favorites.getList(USEREMAIL,USERPASSWD);
	* </code>
	* @see <a href="#list">list</a>
	* @see <a href="#onGetList">onGetList</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.favorites.getList.html">flickr.favorites.getList at Flickr API page</a>
	* */
	public function getList (user_email:String,user_password:String,user_id:String, per_page:String, page:String)
	{
		var flickrFavoritesObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.favorites.getList&api_key="+this._api_key;
		if(user_email) this._email = user_email;
		if(user_password) this._password = user_password;
		if ((this._email)&&(this._password))
		{
			method_url +="&email="+this._email+"&password="+this._password;
		}
		if(user_id) method_url +="&user_id="+user_id;
		if(per_page) method_url +="&per_page="+per_page;
		if(page) method_url +="&page="+page;
		this._response.onLoad = function(success){
			trace(this)
			if(success){
				var isRsp = false;
				var plist = [];
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="photos"){
								var page = rspChild.attributes['page']
								var perpage = rspChild.attributes['perpage']
								var index = page*perpage-perpage;
								for(var photosChild = rspChild.firstChild; photosChild != null; photosChild = photosChild.nextSibling){
									if(photosChild.nodeName=="photo"){
										var entry = new Object();
										entry.id = photosChild.attributes['id']
										entry.owner = photosChild.attributes['owner']
										entry.title = photosChild.attributes['title']
										entry.ispublic = photosChild.attributes['ispublic']
										entry.isfriend = photosChild.attributes['isfriend']
										entry.isfamily = photosChild.attributes['isfamily']
										flickrFavoritesObjPointer._list[index] = entry
										index++;
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrFavoritesObjPointer.onGetList(error,flickrFavoritesObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrFavoritesObjPointer.onGetList(error,flickrFavoritesObjPointer)
				}
				if(!error){
					flickrFavoritesObjPointer.onGetList(0,flickrFavoritesObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrFavoritesObjPointer.onGetList(error,flickrFavoritesObjPointer)
			}
		}
		this._response.load(method_url);
	};
	/**
	* Adds a photo to a user's favorites list.
	* OBS: This method couldnt be called "add" beacause that's a reserved keyword in Flash
	* @param user_email:	(Required) The email used to authenticate on Flickr 
	* @param user_password:	(Required) The password used to authenticate on Flickr
	* @param photo_id:		(Required) The id of the photo to add to the user's favorites.
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.favorites.onAdd = function(error){
	*	if(!error){
	*	 	trace("photo successful added")
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.favorites.addPhoto(USEREMAIL,USERPASSWD,PHOTOID);
	* </code>
	* @see <a href="#onAdd">onAdd</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.favorites.add.html">flickr.favorites.add at Flickr API page</a>
	* */
	public function addPhoto(user_email:String,user_password:String,photo_id:String)
	{
		addOrRemove(user_email,user_password,photo_id,"add")
	}
	/**
	* Removes a photo from a user's favorites list.
	* @param user_email:	(Required) The email used to authenticate on Flickr 
	* @param user_password:	(Required) The password used to authenticate on Flickr
	* @param photo_id:		(Required) The id of the photo to add to the user's favorites.
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.favorites.onRemove = function(error){
	*	if(!error){
	*	 	trace("photo successful removed")
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.favorites.remove(USEREMAIL,USERPASSWD,PHOTOID);
	* </code>
	* @see <a href="#onRemove">onRemove</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.favorites.remove.html">flickr.favorites.remove at Flickr API page</a>
	* */
	public function remove(user_email:String,user_password:String,photo_id:String)
	{
		addOrRemove(user_email,user_password,photo_id,"remove")
	}
	
	private function addOrRemove(user_email:String,user_password:String,photo_id:String,method:String)
	{
		if(method=="add"){
			var callback = this.onAdd
		}else if(method == "remove"){
			var callback = this.onRemove
		}
		var method_url = _rest_endpoint+"?method=flickr.favorites."+method+"&api_key="+this._api_key;
		if(user_email) this._email = user_email;
		if(user_password) this._password = user_password;
		if ((this._email)&&(this._password)&&(photo_id))
		{
			method_url +="&email="+this._email+"&password="+this._password+"&photo_id="+photo_id;
			this._response.onLoad = function(success){
				if(success){
					trace(this)
					var isRsp = false;
					var error = 0;
					for(var node = this.firstChild; node != null; node = node.nextSibling){
						if(node.nodeName == "rsp"){
							isRsp = true;
							for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
								if(rspChild.nodeName=="err"){
									error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
									callback(error)
								}
							}
						}
					}
					if(!isRsp){
						error ="ERROR: Response not in REST format";
						callback(error)
					}
					if(!error){
						callback(0)
					}					
				}else{
					
				}
			}
			this._response.load(method_url)
		}
		
	}	
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}
	/**
	* a list of photos, resulted from a getList method call
	* is an Array of objects, each object has the following attributes
	* id, owner, title, ispublic, isfriend, isfamily
	* @see <a href="#getList">getList</a>
	* */
	public function get list():Array
	{
		return this._list;
	}
};
