﻿/**
* FlickrPeople
* last update 08/28/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrPeople
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _rest_endpoint : String;
	private var _response : XML;
	private var _user_id : String;
	private var _username: String;
	private var _isadmin: Boolean;
	private var _ispro: Boolean;
	private var _realname: String;
	private var _location: String;
	private var _firstdate: Date;
	private var _count: Number;
	private var _onlineList: Array;
	private var _publicPhotos: Array;

	/** 
	* callback function, called at the end of a findByEmail, or findByUsername method
	* @see <a href="#findByEmail">findByEmail</a>
	* @see <a href="#findByUsername">findByUsername</a>
	* */
	public var onFinded:Function;
	/** 
	* callback function, called at the end of a getInfo method
	* @see <a href="#getInfo">getInfo</a>
	* */
	public var onInfo:Function;
	/** 
	* callback function, called at the end of a getOnlineList method
	* @see <a href="#getOnlineList">getOnlineList</a>
	* */
	public var onOnlineList:Function;
	/** 
	* callback function, called at the end of a getPublicPhotos method
	* @see <a href="#getPublicPhotos">getPublicPhotos</a>
	* */
	public var onPublicPhotos:Function;
	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrPeople object, you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrPeople (api_key:String, rest_endpoint:String){
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};		
		this._response = new XML()
		this._response.ignoreWhite = true;
	}
		
	/**
	* Return a user's NSID, given their email address
	* the results are stored in the user_id and username attributes
	* @param find_email :	The email used to find user nsid and username
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.people.onFinded = function(error,obj){
	*	if(!error){
	*	 	trace(obj.username)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.people.findByEmail("someemail@somedomain.com");
	* </code>
	* @see <a href="#user_id">user_id</a>
	* @see <a href="#username">username</a>
	* @see <a href="#onFinded">onFinded</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.people.findByEmail.html">flickr.people.findByEmail.html at Flickr API page</a>
	* */
	public function findByEmail(find_email:String){
		this._findBy(find_email, 0)
	}
	/**
	* Return a user's NSID, given their username.
	* the results are stored in the user_id and username attributes
	* @param username :	The username used to find user nsid and username
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.people.onFinded = function(error,obj){
	*	if(!error){
	* 		trace(obj.username)
	*	 	trace(obj.user_id)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.people.findByUsername("someusername");
	* </code>
	* @see <a href="#user_id">user_id</a>
	* @see <a href="#username">username</a>
	* @see <a href="#onFinded">onFinded</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.people.findByUsername.html">flickr.people.findByUsername.html at Flickr API page</a>
	* */
	public function findByUsername(username:String){
		trace(username)
		this._findBy(username, 1)
	}
	// findByEmail and findByUsername have identical responses, this flag is to be able
	// to use the same code for both functions with minimum changes
	private function _findBy(find_argument:String, flag:Number){
		var flickrPeopleObjPointer = this;
		//findByEmail
		if(flag==0){
			var method_url = _rest_endpoint+"?method=flickr.people.findByEmail&api_key="+this._api_key;
			method_url+="&find_email="+find_argument;
		//findByUsername
		}else if(flag==1){
			var method_url = _rest_endpoint+"?method=flickr.people.findByUsername&api_key="+this._api_key;
			method_url+="&username="+find_argument;
		} 
		this._response.onLoad = function(success){
			if(success){
				var isRsp = false;
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="user"){
								var entry = new Object();
								flickrPeopleObjPointer._user_id = rspChild.attributes['id']
								for(var user = rspChild.firstChild; user != null; user = user.nextSibling){
									if(user.nodeName=="username"){
										if(user.firstChild.nodeType==3){
											flickrPeopleObjPointer._username = user.firstChild.nodeValue;
										}
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrPeopleObjPointer.onFinded(error,flickrPeopleObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrPeopleObjPointer.onFinded(error,flickrPeopleObjPointer)
				}
				if(!error){
					flickrPeopleObjPointer.onFinded(0,flickrPeopleObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrPeopleObjPointer.onFinded(error,flickrPeopleObjPointer)
			}
		}
		this._response.load(method_url);
	}
	/**
	* Get information about a user
	* the results are stored in the attributes: username, realname, location, firstDate, count, isadmin and ispro
	* @param user_id :	The NSID of the user to fetch information about
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.people.onInfo = function(error,obj){
	* 	if(!error){
	* 		trace(obj.username)
	* 		trace(obj.realname)
	* 		trace(obj.location)
	* 		trace(obj.firstdate)
	* 		trace(obj.count)
	* 		trace(obj.isadmin)
	* 		trace(obj.ispro)
 	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.people.getInfo("49503114626@N01");
	* </code>
	* @see <a href="#username">username</a>
	* @see <a href="#realname">realname</a>
	* @see <a href="#location">location</a>
	* @see <a href="#firstdate">firstdate</a>
	* @see <a href="#count">count</a>
	* @see <a href="#isadmin">isadmin</a>
	* @see <a href="#ispro">ispro</a>
	* @see <a href="#onInfo">onInfo</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.people.getInfo.html">flickr.people.getInfo.html at Flickr API page</a>
	* */
	public function getInfo(user_id:String){
		var flickrPeopleObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.people.getInfo&api_key="+this._api_key+"&user_id="+user_id;
		this._response.onLoad = function(success){
			if(success){
				var isRsp = false;
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="person"){
								flickrPeopleObjPointer._isadmin = rspChild.attributes['isadmin'];
								flickrPeopleObjPointer._ispro = rspChild.attributes['ispro'];
								for(var personChild = rspChild.firstChild; personChild != null; personChild = personChild.nextSibling){
									if(personChild.nodeName=="username"){
										if(personChild.firstChild.nodeType==3){
											flickrPeopleObjPointer._username = personChild.firstChild.nodeValue;
										}
									}
									if(personChild.nodeName=="realname"){
										if(personChild.firstChild.nodeType==3){
											flickrPeopleObjPointer._realname = personChild.firstChild.nodeValue;
										}
									}
									if(personChild.nodeName=="location"){
										if(personChild.firstChild.nodeType==3){
											flickrPeopleObjPointer._location = personChild.firstChild.nodeValue;
										}
									}
									if(personChild.nodeName=="photos"){
										for(var photosChild = personChild.firstChild; photosChild != null; photosChild = photosChild.nextSibling){
											if(photosChild.nodeName=="firstdate"){
												if(photosChild.firstChild.nodeType==3){
													flickrPeopleObjPointer._firstdate = new Date()
													flickrPeopleObjPointer._firstdate.setTime(photosChild.firstChild.nodeValue)
												}
											}
											if(photosChild.nodeName=="count"){
												if(photosChild.firstChild.nodeType==3){
													flickrPeopleObjPointer._count = photosChild.firstChild.nodeValue;
												}
											}
										}
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrPeopleObjPointer.onInfo(error,flickrPeopleObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrPeopleObjPointer.onInfo(error,flickrPeopleObjPointer)
				}
				if(!error){
					flickrPeopleObjPointer.onInfo(0,flickrPeopleObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrPeopleObjPointer.onInfo(error,flickrPeopleObjPointer)
			}
		}
		this._response.load(method_url);
		
	}
	/**
	* Get a list of all online users.
	* the results are stored in the onlineList attribute
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.people.onOnlineList = function(error,obj){
	*	if(!error){
	*	 	trace(obj.onlineList[0].id)
	*	 	trace(obj.onlineList[0].username)
	*	 	trace(obj.onlineList[0].online)
	*	 	trace(obj.onlineList[0].away_msg)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.people.getOnlineList();
	* </code>
	* @see <a href="#onlineList">onlineList</a>
	* @see <a href="#onOnlineList">onOnlineList</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.people.getOnlineList.html">flickr.people.getOnlineList at Flickr API page</a>
	* */
	public function getOnlineList ()
	{
		var flickrPeopleObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.people.getOnlineList&api_key="+this._api_key;
		this._response.onLoad = function(success){
			trace(this)
			if(success){
				var isRsp = false;
				var olist = [];
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="online"){
								for(var onlineChild = rspChild.firstChild; onlineChild != null; onlineChild = onlineChild.nextSibling){
									if(onlineChild.nodeName=="user"){
										var entry = new Object();
										entry.id = onlineChild.attributes['nsid']
										entry.username = onlineChild.attributes['username']
										entry.online = onlineChild.attributes['online']
										if(onlineChild.firstChild.nodeType==3){
											//NEED TEST, I'm not certain that it will work
											entry.away_msg = onlineChild.firstChild.nodeValue;
										}
										olist.push(entry)
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrPeopleObjPointer.onOnlineList(error,flickrPeopleObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrPeopleObjPointer.onOnlineList(error,flickrPeopleObjPointer)
				}
				if(!error){
					flickrPeopleObjPointer._onlineList = olist;
					flickrPeopleObjPointer.onOnlineList(0,flickrPeopleObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrPeopleObjPointer.onOnlineList(error,flickrPeopleObjPointer)
			}
		}
		this._response.load(method_url);
	};

	/**
	* Get a list of public photos for the given user.
	* the results are stored in the publicPhotos attribute
	* @param user_id :	(required) The NSID of the user who's photos to return.
	* @param per_page :	(optional) Number of photos to return per page. If this argument is ommited, it defaults to 100. The maximum allowed value is 500.
	* @param page :		(optional) The page of results to return. If this argument is ommited, it defaults to 1.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.people.onPublicPhotos = function(error,obj){
	*	if(!error){
	*	 	trace(obj.publicPhotos[0].id)
	*	 	trace(obj.publicPhotos[0].owner)
	*	 	trace(obj.publicPhotos[0].title)
	*	 	trace(obj.publicPhotos[0].ispublic)
	*	 	trace(obj.publicPhotos[0].isfriend)
	*	 	trace(obj.publicPhotos[0].isfamily)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.people.getPublicPhotos(USERID);
	* </code>
	* @see <a href="#publicPhotos">publicPhotos</a>
	* @see <a href="#onPublicPhotos">onPublicPhotos</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.people.getPublicPhotos.html">flickr.people.getPublicPhotos at Flickr API page</a>
	* */
	public function getPublicPhotos (user_id:String, per_page:Number, page:Number)
	{
		var flickrPeopleObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.people.getPublicPhotos&api_key="+this._api_key;
		if(user_id){
			method_url +="&user_id="+user_id;
		}else{
			trace("ERROR: user_id is required")
			return
		}
		if(per_page){
			method_url +="&per_page="+per_page;
		}
		if(page){
			method_url +="&page="+page;
		}
		this._response.onLoad = function(success){
			trace(this)
			if(success){
				var isRsp = false;
				var plist = [];
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="photos"){
								var page = rspChild.attributes['page']
								var perpage = rspChild.attributes['perpage']
								var index = page*perpage-perpage;
								trace(index)
								for(var photosChild = rspChild.firstChild; photosChild != null; photosChild = photosChild.nextSibling){
									if(photosChild.nodeName=="photo"){
										var entry = new Object();
										entry.id = photosChild.attributes['id']
										entry.owner = photosChild.attributes['owner']
										entry.title = photosChild.attributes['title']
										entry.ispublic = photosChild.attributes['ispublic']
										entry.isfriend = photosChild.attributes['isfriend']
										entry.isfamily = photosChild.attributes['isfamily']
										flickrPeopleObjPointer._publicPhotos[index] = entry
										index++;
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrPeopleObjPointer.onPublicPhotos(error,flickrPeopleObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrPeopleObjPointer.onPublicPhotos(error,flickrPeopleObjPointer)
				}
				if(!error){
					flickrPeopleObjPointer.onPublicPhotos(0,flickrPeopleObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrPeopleObjPointer.onPublicPhotos(error,flickrPeopleObjPointer)
			}
		}
		this._response.load(method_url);
	};
	/**
	*@see <a href="#findByEmail">findByEmail</a>
	*@see <a href="#findByUsername">findByUsername</a>
	*@see <a href="#getInfo">getInfo</a>
	*/
	public function get username():String
	{
		return this._username;
	}
	/**
	*@see<a href="#findByEmail">findByEmail</a>
	*@see<a href="#findByUsername">findByUsername</a>
	* */
	public function get user_id():String
	{
		return this._user_id;
	}
	/**@see <a href="#getInfo">getInfo</a>
	* */
	public function get isadmin(): Boolean
	{
		return this._isadmin;
	}
	/**@see <a href="#getInfo">getInfo</a>
	* */
	public function get ispro(): Boolean
	{
		return this._ispro;
	}
	/**@see <a href="#getInfo">getInfo</a>
	* */
	public function get realname(): String
	{
		return this._realname;
	}
	/**@see <a href="#getInfo">getInfo</a>
	* */
	public function get location(): String
	{
		return this._location;
	}
	/**@see <a href="#getInfo">getInfo</a>
	* */
	public function get firstdate(): Date
	{
		return this._firstdate;
	}
	/**@see <a href="#getInfo">getInfo</a>
	* */
	public function get count(): Number
	{
		return this._count;
	}
	/**
	* a list of the online users, resulted from a getOnlineList method call
	* is an Array of objects, each object has the following attributes
	* id, username, online, away_msg
	* @see <a href="#getOnlineList">getOnlineList</a>
	* */
	public function get onlineList(): Array
	{
		return this._onlineList;
	}
	/**
	* a list of photos, resulted from a getPublicPhotos method call
	* is an Array of objects, each object has the following attributes
	* id, owner, title, ispublic, isfriend, isfamily
	* @see <a href="#getPublicPhotos">getPublicPhotos</a>
	* */
	public function get publicPhotos(): Array
	{
		return this._publicPhotos;
	}
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}	
}