﻿/**
* FlickrPhotos
* last update 09/02/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrPhotos
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _email : String;
	private var _password : String;
	private var _rest_endpoint : String;
	private var _response : XML;
	private var _photo_info : Object;
	private var _sizes : Array;
	/** 
	* callback function, called at the end of a getInfo method
	* @see <a href="#getInfo">getInfo</a>
	* */
	public var onGetInfo:Function;
	/** 
	* callback function, called at the end of a getSizess method
	* @see <a href="#getSizess">getSizess</a>
	* */
	public var onGetSizes:Function;
	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrPhotos, object you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param user_email:	(optional) The email to use on methods that requires user authentication 
	* @param user_password:	(optional) The email to use on methods that requires user authentication 
	* @see <a href="http://www.flickr.com/services/api/misc.userauth.html">User Authentication Flickr API page</a>
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrPhotos (api_key : String, user_email : String, user_password : String,rest_endpoint : String)
	{
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};
		this._response = new XML()
		this._response.ignoreWhite = true;
		this._email = user_email;
		this._password = user_password;
	};
	/**
	* Get information about a photo.
	* photo_info:
	* dateuploaded, ownerId, ownerUsername, ownerRealName, ownerLocation, title, description, ispublic,
	* isfamily, isfriend, cancomment, canaddmeta, comments, notes (id, author, authorname, x, y, w, h,note), tags
	* @param user_email:	(Required) The email used to authenticate on Flickr 
	* @param user_password:	(Required) The password used to authenticate on Flickr
	* @param photo_id:		(Required) The is of the photo to fetch information for. 
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.photos.onGetInfo = function(error,obj){
	*	if(!error){
	*	 	trace(obj.photo_info.dateuploaded)
	*	 	trace(obj.photo_info.ownerId)
	*	 	trace(obj.photo_info.ownerUsername)
	*	 	trace(obj.photo_info.ownerRealName)
	*	 	trace(obj.photo_info.ownerLocation)
	*	 	trace(obj.photo_info.title)
	*	 	trace(obj.photo_info.description)
	*	 	trace(obj.photo_info.ispublic)
	*	 	trace(obj.photo_info.isfamily)
	*	 	trace(obj.photo_info.isfriend)
	*	 	trace(obj.photo_info.cancomment)
	*	 	trace(obj.photo_info.canaddmeta)
	*	 	trace(obj.photo_info.comments)
	*	 	trace(obj.photo_info.notes[0].id)
	*	 	trace(obj.photo_info.notes[0].author)
	*	 	trace(obj.photo_info.notes[0].authorname)
	*	 	trace(obj.photo_info.notes[0].x)
	*	 	trace(obj.photo_info.notes[0].y)
	*	 	trace(obj.photo_info.notes[0].w)
	*	 	trace(obj.photo_info.notes[0].h)
	*	 	trace(obj.photo_info.notes[0].note)
	*	 	trace(obj.photo_info.tags[0])
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.photos.getInfo(PHOTOID);
	* </code>
	* @see <a href="#photoInfo">photoInfo</a>
	* @see <a href="#onGetInfo">onGetInfo</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.photos.getInfo.html">flickr.photos.getInfo at Flickr API page</a>
	* */
	public function getInfo (photo_id:String)
	{
		var flickrPhotosObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.photos.getInfo&api_key="+this._api_key;
		if(!photo_id) {
			trace("error")
		}else{
			method_url +="&photo_id="+photo_id;	
			this._response.onLoad = function(success){
				trace(this)
				if(success){
					var isRsp = false;
					var pInfo = new Object();
					pInfo.notes = [];
					pInfo.tags = [];
					for(var node = this.firstChild; node != null; node = node.nextSibling){
						if(node.nodeName == "rsp"){
							isRsp = true;
							for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
								if(rspChild.nodeName=="photo"){
									pInfo.dateuploaded = rspChild.attributes['dateuploaded']
									for(var photoChild = rspChild.firstChild; photoChild != null; photoChild = photoChild.nextSibling){
										if(photoChild.nodeName=="owner"){
											pInfo.ownerId = photoChild.attributes['nsid']
											pInfo.ownerUsername = photoChild.attributes['username']
											pInfo.ownerRealName = photoChild.attributes['realname']
											pInfo.ownerLocation = photoChild.attributes['location']
										}else if(photoChild.nodeName=="title"){
											pInfo.title = photoChild.firstChild.nodeValue;
										}else if(photoChild.nodeName=="description"){
											pInfo.description = photoChild.firstChild.nodeValue;
										}else if(photoChild.nodeName=="visibility"){
											pInfo.ispublic = photoChild.attributes['ispublic']
											pInfo.isfriend = photoChild.attributes['isfriend']
											pInfo.isfamily = photoChild.attributes['isfamily']
										}else if(photoChild.nodeName=="editability"){
											pInfo.cancomment = photoChild.attributes['cancomment']
											pInfo.canaddmeta = photoChild.attributes['canaddmeta']
										}else if(photoChild.nodeName=="comments"){
											pInfo.comments = photoChild.firstChild.nodeValue;
										}else if(photoChild.nodeName=="notes"){
											for(var notesChild = photoChild.firstChild; notesChild != null; notesChild = notesChild.nextSibling){
												if(notesChild.nodeName=="note"){
													var obj = new Object()
													obj.id = notesChild.attributes['id']
													obj.author = notesChild.attributes['author']
													obj.authorname = notesChild.attributes['authorname']
													obj.x = notesChild.attributes['x']
													obj.y = notesChild.attributes['y']
													obj.w = notesChild.attributes['w']
													obj.h = notesChild.attributes['h']
													obj.note = notesChild.firstChild.nodeValue;
													pInfo.notes.push(obj)
												}
											}
										}else if(photoChild.nodeName=="tags"){
											for(var tagsChild = photoChild.firstChild; tagsChild != null; tagsChild = tagsChild.nextSibling){
												pInfo.tags.push(tagsChild.firstChild.nodeValue)
											}
										}
									}
								}
								if(rspChild.nodeName=="err"){
									var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
									flickrPhotosObjPointer.onGetInfo(error,flickrPhotosObjPointer)
								}
							}
						}
					}
					if(!isRsp){
						var error ="ERROR: Response not in REST format";
						flickrPhotosObjPointer.onGetInfo(error,flickrPhotosObjPointer)
					}
					if(!error){
						flickrPhotosObjPointer._photo_info = pInfo;
						flickrPhotosObjPointer.onGetInfo(0,flickrPhotosObjPointer)
					}
				}else{
					var error ="ERROR: Could not load url:"+method_url;
					flickrPhotosObjPointer.onGetInfo(error,flickrPhotosObjPointer)
				}
			}
			this._response.load(method_url);
		}
	};
	
	/**
	* Returns the available sizes for a photo.
	* sizes:
	* label, width, height, source, url
	* @param photo_id:		(Required) The id of the photo to fetch size information for.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.photos.onGetSizes = function(error,obj){
	*	if(!error){
	*	 	trace(obj.sizes[0].label)
	*	 	trace(obj.sizes[0].width)
	*	 	trace(obj.sizes[0].height)
	*	 	trace(obj.sizes[0].source)
	*	 	trace(obj.sizes[0].url)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.photos.getSizes(PHOTOID);
	* </code>
	* @see <a href="#sizes">sizes</a>
	* @see <a href="#onGetSizes">onGetSizes</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.photos.getSizes.html">flickr.photos.getSizes at Flickr API page</a>
	* */
	public function getSizes (photo_id:String)
	{
		var flickrPhotosObjPointer = this;
		var method_url = _rest_endpoint+"?method=flickr.photos.getSizes&api_key="+this._api_key;
		if(!photo_id) {
			trace("error")
		}else{
			method_url +="&photo_id="+photo_id;	
			this._response.onLoad = function(success){
				trace(this)
				if(success){
					var isRsp = false;
					var sizes = [];
					for(var node = this.firstChild; node != null; node = node.nextSibling){
						if(node.nodeName == "rsp"){
							isRsp = true;
							for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
								if(rspChild.nodeName=="sizes"){
									for(var sizesChild = rspChild.firstChild; sizesChild != null; sizesChild = sizesChild.nextSibling){
										if(sizesChild.nodeName=="size"){
											var obj = new Object();
											obj.label = sizesChild.attributes['label']
											obj.width = sizesChild.attributes['width']
											obj.height = sizesChild.attributes['height']
											obj.source = sizesChild.attributes['source']
											obj.url = sizesChild.attributes['url']
											sizes.push(obj)
										}
									}
								}
								if(rspChild.nodeName=="err"){
									var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
									flickrPhotosObjPointer.onGetSizes(error,flickrPhotosObjPointer)
								}
							}
						}
					}
					if(!isRsp){
						var error ="ERROR: Response not in REST format";
						flickrPhotosObjPointer.onGetSizes(error,flickrPhotosObjPointer)
					}
					if(!error){
						flickrPhotosObjPointer._sizes = sizes;
						flickrPhotosObjPointer.onGetSizes(0,flickrPhotosObjPointer)
					}
				}else{
					var error ="ERROR: Could not load url:"+method_url;
					flickrPhotosObjPointer.onGetSizes(error,flickrPhotosObjPointer)
				}
			}
			this._response.load(method_url);
		}
	};
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}
	public function get photo_info():Object
	{
		return this._photo_info;
	}
	public function get sizes():Array
	{
		return this._sizes;
	}
};
