﻿/**
* FlickrTags
* last update 08/28/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrTags
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _rest_endpoint : String;
	private var _tags : Array;
	private var _response : XML;

	/** 
	* callback function, called at the end of a getListPhoto method
	* @see <a href="#getListPhoto">getListPhoto</a>
	* */	
	public var onListPhoto:Function;
	/** 
	* callback function, called at the end of a getListUser method
	* @see <a href="#getListUser">getListUser</a>
	* */
	public var onListUser:Function;
	/** 
	* callback function, called at the end of a getListUserPopular method
	* @see <a href="#getListUser">getListUserPopular</a>
	* */
	public var onListUserPopular:Function;
	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrTags object, you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrTags (api_key:String, rest_endpoint:String){
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};		
		this._response = new XML()
		this._response.ignoreWhite = true;
	}
	/**
	* Get the tag list for a given photo.
	* the results are stored in the list attribute
	* @param photo_id :		(required) The id of the photo to return tags for.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.tags.onListPhoto = function(error,obj){
	*	if(!error){
	*	 	trace(obj.list[0].tag)
	*	 	trace(obj.list[0].author)
	*	 	trace(obj.list[0].authorname)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.tags.getListPhoto(PHOTOID);
	* </code>
	* @see <a href="#list">list</a>
	* @see <a href="#onListPhoto">onListPhoto</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.tags.getListPhoto.html">flickr.tags.getListPhoto at Flickr API page</a>
	* */
	public function getListPhoto (photo_id:String)
	{
		if(!photo_id){
			//error
		}else{
			getTagList ("getListPhoto",null,photo_id)
		}
	};
	/**
	* Get the tag list for a given user (or the currently logged in user).
	* the results are stored in the list attribute
	* @param user_id :		(opitional) The NSID of the user to fetch the tag list for. 
	* 						If this argument is not specified, the currently logged in user (if any) is assumed.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.tags.onListUser = function(error,obj){
	*	if(!error){
	*	 	trace(obj.list[0])
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.tags.getListUser(USERID);
	* </code>
	* @see <a href="#list">list</a>
	* @see <a href="#onListUser">onListUser</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.tags.getListUser.html">flickr.tags.getListUser at Flickr API page</a>
	* */
	public function getListUser (user_id:String)
	{
		getTagList("getListUser",user_id)
	};
	/**
	* Get the popular tags for a given user (or the currently logged in user).
	* the results are stored in the list attribute
	* @param user_id :		(opitional) The NSID of the user to fetch the tag list for. 
	* 						If this argument is not specified, the currently logged in user (if any) is assumed.
	* @param count :		(opitional) Number of popular tags to return. defaults to 10 when this argument is not present.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.tags.onListUserPopular = function(error,obj){
	*	if(!error){
	*	 	trace(obj.list[2].tag)
	*	 	trace(obj.list[2].count)
	* 	}else{
	*		trace(error)
	*	}
	* };
	* my_flickr.tags.getListUserPopular(USERID,3);
	* </code>
	* @see <a href="#list">list</a>
	* @see <a href="#onListUserPopular">onListUserPopular</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.tags.getListUserPopular.html">flickr.tags.getListUserPopular at Flickr API page</a>
	* */
	public function getListUserPopular (user_id:String,count:Number)
	{
		getTagList("getListUserPopular",user_id,null,count)
	};
	
	private function getTagList (method:String,user_id:String,photo_id:String,count:Number)
	{
		var flickrTagsObjPointer = this;
		if(method=="getListPhoto"){
			var callback = this.onListPhoto
		}
		if(method=="getListUser"){
			var callback = this.onListUser
		}
		if(method=="getListUserPopular"){
			var callback = this.onListUserPopular
		}
		var method_url = _rest_endpoint+"?method=flickr.tags."+method+"&api_key="+this._api_key;
		if(user_id){
			method_url += "&user_id="+user_id
		}
		if(photo_id){
			method_url += "&photo_id="+photo_id
		}
		if(count){
			method_url += "&count="+count
		}
		this._response.onLoad = function(success){
			if(success){
				trace(this)
				var isRsp = false;
				var tags = [];
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if((rspChild.nodeName=="photo")||(rspChild.nodeName=="who")){
								for(var photoChild = rspChild.firstChild; photoChild != null; photoChild = photoChild.nextSibling){
									if(photoChild.nodeName=="tags"){
										for(var tagsChild = photoChild.firstChild; tagsChild != null; tagsChild = tagsChild.nextSibling){
											if(tagsChild.nodeName=="tag"){
												var entry = new Object();
												var hasattributes = false;
												for(var i in tagsChild.attributes){
													entry[i] = tagsChild.attributes[i];
													hasattributes = true;
												}
												if(tagsChild.firstChild.nodeType==3){
													if(hasattributes){
														entry.tag = tagsChild.firstChild.nodeValue;
													}else{
														entry = tagsChild.firstChild.nodeValue;								
													}
												}
												tags.push(entry)
											}
										}
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								callback(error,flickrTagsObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					callback(error,flickrTagsObjPointer)
				}
				if(!error){
					flickrTagsObjPointer._tags = tags;
					callback(0,flickrTagsObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				callback(error,flickrTagsObjPointer)
			}
		}
		trace("abrindo "+method_url)
		this._response.load(method_url);
	};

	/**
	* the last tags list returned from server
	* if it was the result of a getListUser call list is an Array containing the tags strings
	* if it was the result of a getListUserPopular call list is an Array of objects, each object having the following attributes:
	* tag, count
	* if it was the result of a getListPhoto call list is an Array of objects, each object having the following attributes:
	* tag, author, authorname
	*@see<a href="#getListPhoto">getListPhoto</a>
	*@see<a href="#getListUser">getListUser</a>
	*@see<a href="#getListUserPopular">getListUserPopular</a>
	* */
	public function get list():Array
	{
		return this._tags;
	}
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}	
}