﻿/**
* FlickrTest
* last update 08/29/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrTest
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _rest_endpoint : String;
	private var _username : String;
	private var _user_id : String;
	private var _params : Array;
	private var _response : XML;

	/** 
	* callback function, called at the end of a echo method
	* @see <a href="#echo">echo</a>
	* */
	public var onEcho:Function;
	/** 
	* callback function, called at the end of a login method
	* @see <a href="#login">login</a>
	* */
	public var onTestLogin:Function;
	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrTags object, you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrTest (api_key:String, rest_endpoint:String){
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};		
		this._response = new XML()
		this._response.ignoreWhite = true;
		this._params = new Array()
	}
	
	/**
	* A testing method which echo's all paramaters back in the response.
	* the results are stored in the params attribute
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.test.onEcho = function(error,obj){
	* 	if(!error){
	* 		trace(obj.params[2].name)
	* 		trace(obj.params[2].value)
	* 		trace(obj.params[3].name)
	* 		trace(obj.params[3].value)
	* 	}else{
	* 		trace(error)
	* 	}
	* };
	* my_flickr.test.echo({test:1,foo:"bar"});
	* </code>
	* @see <a href="#params">params</a>
	* @see <a href="#onEcho">onEcho</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.test.echo.html">flickr.test.echo at Flickr API page</a>
	* */
	public function echo (params)
	{
		var method_url = _rest_endpoint+"?method=flickr.test.echo&api_key="+this._api_key;
		for(var i in params){
			method_url += "&"+i+"="+params[i]
		}
		var flickrTestObjPointer = this;
		this._response.onLoad = function(success){
			if(success){
				var isRsp = false;
				var params = [];
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							var entry = new Object();
							entry.name = rspChild.nodeName;
							entry.value = rspChild.firstChild.nodeValue;
							params.push(entry)
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrTestObjPointer.onEcho(error,flickrTestObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrTestObjPointer.onEcho(error,flickrTestObjPointer)
				}
				if(!error){
					flickrTestObjPointer._params = params;
					flickrTestObjPointer.onEcho(0,flickrTestObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrTestObjPointer.onEcho(error,flickrTestObjPointer)
			}
		}
		this._response.load(method_url);
	};
	/**
	* A testing method which checks if the caller is logged in then returns their username.
	* the results are stored in the attributes user_id and username
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.test.onTestLogin = function(error,obj){
	* 	if(!error){
	* 		trace(obj.user_id)
	* 		trace(obj.username)
	* 	}else{
	* 		trace(error)
	* 	}
	* };
	* my_flickr.test.login();
	* </code>
	* @see <a href="#user_id">user_id</a>
	* @see <a href="#username">username</a>
	* @see <a href="#onTestLogin">onTestLogin</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.test.login.html">flickr.test.login at Flickr API page</a>
	* */
	public function login ()
	{
		var method_url = _rest_endpoint+"?method=flickr.test.login&api_key="+this._api_key;
		var flickrTestObjPointer = this;
		this._response.onLoad = function(success){
			if(success){
				trace(this)
				var isRsp = false;
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if (rspChild.nodeName=="user"){
								trace("user")
								flickrTestObjPointer._user_id = rspChild.attributes['id']
								for(var userChild = rspChild.firstChild; userChild != null; userChild = userChild.nextSibling){
									if(userChild.nodeName=="username"){
										trace("username")
										flickrTestObjPointer._username = userChild.firstChild.nodeValue;
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrTestObjPointer.onTestLogin(error,flickrTestObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrTestObjPointer.onTestLogin(error,flickrTestObjPointer)
				}
				if(!error){
					flickrTestObjPointer.onTestLogin(0,flickrTestObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrTestObjPointer.onTestLogin(error,flickrTestObjPointer)
			}
		}
		this._response.load(method_url);
	};

	/**
	* a list of parameters, resulted from a echo method call
	* is an Array of objects, each object has the following attributes
	* name, value
	*@see<a href="#echo">echo</a>
	* */
	public function get params():Array
	{
		return this._params;
	}
	public function get user_id():String
	{
		return this._user_id;
	}
	public function get username():String
	{
		return this._username;
	}
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}	
}