﻿/**
* FlickrUrls
* last update 08/29/2004
* @version	1.0
* @author	Fabricio Zuardi
*/;
class com.zuardi.flickr.FlickrUrls
{
	private static var REST_ENDPOINT : String = "http://www.flickr.com/services/rest/";
	private var _api_key : String;
	private var _rest_endpoint : String;
	private var _group_url : String;
	private var _group_name : String;
	private var _user_photos_url : String;
	private var _user_profile_url : String;
	private var _group_id : String;
	private var _user_id : String;
	
	private var _response : XML;

	/** 
	* callback function, called at the end of a getGroup method
	* @see <a href="#getGroup">getGroup</a>
	* */
	public var onGetGroup:Function;
	/** 
	* callback function, called at the end of a getUserPhotos method
	* @see <a href="#getUserPhotos">getUserPhotos</a>
	* */
	public var onGetUserPhotos:Function;
	/**
	* These parameters are filled automatically if you are using a <a href="com_zuardi_flickr_Flickr.html">Flickr object</a>
	* otherwise, if you are instantiating a FlickrTags object, you have to enter the parameters
	* @param api_key:		(required) The api_key of the application
	* @param rest_endpoint:	(optional) The rest endpoint url
	* */
	function FlickrUrls (api_key:String, rest_endpoint:String){
		if ( ! api_key)
		{
			throw new Error ("api_key is required");
		} else
		{
			this._api_key = api_key;
		};
		if ( ! rest_endpoint)
		{
			this._rest_endpoint = REST_ENDPOINT;
		} else
		{
			this._rest_endpoint = rest_endpoint;
		};	
		trace("aa"+this._rest_endpoint)
		this._response = new XML()
		this._response.ignoreWhite = true;
	}
	
	/**
	* Returns a group NSID, given the url to a group's page or photo pool.
	* the result are stored in the group_id attribute
	* @param url :	(Required)The url to the group's page or photo pool.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.urls.onLookupGroup = function(error,obj){
	* 	if(!error){
	* 		trace(obj.group_id)
	* 	}else{
	* 		trace(error)
	* 	}
	* };
	* my_flickr.urls.lookupGroup(URL);
	* </code>
	* @see <a href="#group_id">group_id</a>
	* @see <a href="#onLookupGroup">onLookupGroup</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.urls.lookupGroup.html">flickr.urls.lookupGroup at Flickr API page</a>
	* */
	public function lookupGroup (url)
	{
		var method_url = _rest_endpoint+"?method=flickr.urls.lookupGroup&api_key="+this._api_key+"&url="+url;
		trace(method_url)
		var flickrUrlsObjPointer = this;
		this._response.onLoad = function(success){
			if(success){
				trace(this)
				var isRsp = false;
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="group"){
								trace(rspChild.attributes['id'])
								flickrUrlsObjPointer._group_id = rspChild.attributes['id']
								for(var groupChild = rspChild.firstChild; groupChild != null; groupChild = groupChild.nextSibling){
									if(groupChild.nodeName=="groupname"){
										trace(groupChild.firstChild.nodeValue)
										flickrUrlsObjPointer._group_name = groupChild.firstChild.nodeValue
									}
								}
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrUrlsObjPointer.onLookupGroup(error,flickrUrlsObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrUrlsObjPointer.onLookupGroup(error,flickrUrlsObjPointer)
				}
				if(!error){
					flickrUrlsObjPointer.onLookupGroup(0,flickrUrlsObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrUrlsObjPointer.onLookupGroup(error,flickrUrlsObjPointer)
			}
		}
		this._response.load(method_url);
	};

	/**
	* Returns the url to a group's page.
	* the result are stored in the group_url attribute
	* @param group_id :	(Required)The NSID of the group to fetch the url for.
	* @usage
	* <code>
	* my_flickr = new Flickr(APIKEY);
	* my_flickr.urls.onGetGroup = function(error,obj){
	* 	if(!error){
	* 		trace(obj.group_url)
	* 	}else{
	* 		trace(error)
	* 	}
	* };
	* my_flickr.urls.getGroup(GROUPID);
	* </code>
	* @see <a href="#group_url">group_url</a>
	* @see <a href="#onGetGroup">onGetGroup</a>
	* @see <a href="#response">response</a>
	* @see <a href="http://www.flickr.com/services/api/flickr.urls.getGroup.html">flickr.urls.getGroup at Flickr API page</a>
	* */
	public function getGroup (group_id)
	{
		var method_url = _rest_endpoint+"?method=flickr.urls.getGroup&api_key="+this._api_key+"&group_id="+group_id;
		var flickrUrlsObjPointer = this;
		this._response.onLoad = function(success){
			if(success){
				trace(this)
				var isRsp = false;
				for(var node = this.firstChild; node != null; node = node.nextSibling){
					if(node.nodeName == "rsp"){
						isRsp = true;
						for(var rspChild = node.firstChild; rspChild != null; rspChild = rspChild.nextSibling){
							if(rspChild.nodeName=="group"){
								flickrUrlsObjPointer._group_url = rspChild.attributes['url']
							}
							if(rspChild.nodeName=="err"){
								var error ="ERROR CODE:"+rspChild.attributes['code']+" msg: "+rspChild.attributes['msg'];
								flickrUrlsObjPointer.onGetGroup(error,flickrUrlsObjPointer)
							}
						}
					}
				}
				if(!isRsp){
					var error ="ERROR: Response not in REST format";
					flickrUrlsObjPointer.onGetGroup(error,flickrUrlsObjPointer)
				}
				if(!error){
					flickrUrlsObjPointer.onGetGroup(0,flickrUrlsObjPointer)
				}
			}else{
				var error ="ERROR: Could not load url:"+method_url;
				flickrUrlsObjPointer.onGetGroup(error,flickrUrlsObjPointer)
			}
		}
		this._response.load(method_url);
	};
	
	public function get group_url():String
	{
		return this._group_url;
	}
	public function get user_photos_url():String
	{
		return this._user_photos_url;
	}
	public function get user_profile_url():String
	{
		return this._user_profile_url;
	}
	public function get group_id():String
	{
		return this._group_id;
	}
	public function get user_id():String
	{
		return this._user_id;
	}
	/**the last xml returned from server*/
	public function get response():XML
	{
		return this._response;
	}	
}